import {
  useCheckboxGroupContext
} from "./chunk-CBD2SE3A.mjs";
import {
  CheckboxIcon
} from "./chunk-AXNKREJM.mjs";
import {
  __name
} from "./chunk-INDHDMU3.mjs";

// src/checkbox.tsx
import { isVNode as _isVNode, createVNode as _createVNode, mergeProps as _mergeProps, Fragment as _Fragment } from "vue";
import { h, defineComponent, computed, onMounted, cloneVNode, watch } from "vue";
import { chakra, omitThemingProps, useMultiStyleConfig } from "@chakra-ui/vue-system";
import { SNAO, vueThemingProps, getValidChildren } from "@chakra-ui/vue-utils";
import * as checkbox from "@zag-js/checkbox";
import { normalizeProps, useMachine, mergeProps } from "@zag-js/vue";
import { useIds } from "@chakra-ui/vue-composables";
import { filterUndefined, mergeWith, pick } from "@chakra-ui/utils";
import { useFormControlContext } from "@chakra-ui/c-form-control";
import { genId } from "@chakra-ui/vue-utils";
function _isSlot(s) {
  return typeof s === "function" || Object.prototype.toString.call(s) === "[object Object]" && !_isVNode(s);
}
__name(_isSlot, "_isSlot");
var CCheckboxControl = chakra("span", {
  baseStyle: {
    display: "inline-flex",
    alignItems: "center",
    justifyContent: "center",
    verticalAlign: "top",
    userSelect: "none",
    flexShrink: 0
  }
});
var CLabel = chakra("label", {
  baseStyle: {
    cursor: "pointer",
    display: "inline-flex",
    alignItems: "center",
    verticalAlign: "top",
    position: "relative"
  }
});
var CCheckbox = defineComponent({
  name: "CCheckbox",
  props: {
    modelValue: {
      type: Boolean,
      default: false
    },
    value: [String, Number],
    id: String,
    isIndeterminate: Boolean,
    isFocusable: Boolean,
    isRequired: Boolean,
    isInvalid: Boolean,
    isDisabled: Boolean,
    isReadonly: Boolean,
    defaultChecked: Boolean,
    name: String,
    "aria-label": String,
    "aria-labelledby": String,
    "aria-invalid": Boolean,
    "aria-describedby": String,
    spacing: {
      type: SNAO,
      default: "0.5rem"
    },
    iconColor: String,
    iconSize: [String, Number],
    icon: Object,
    inputProps: Object,
    ...vueThemingProps
  },
  emits: ["change", "update:modelValue"],
  setup(props, {
    slots,
    attrs,
    emit
  }) {
    const group = useCheckboxGroupContext(computed(() => ({})));
    const ownProps = computed(() => omitThemingProps(props));
    const mergedProps = computed(() => mergeWith({}, group.value, props, attrs));
    const styles = useMultiStyleConfig("Checkbox", mergedProps);
    const inheritedFormControlProps = useFormControlContext(mergedProps);
    const id = genId();
    const [rootId, inputId, controlId, labelId] = useIds(`chakra-checkbox-${id}`, "root", "input", "control", "label");
    const machineContext = computed(() => {
      var _a, _b, _c, _d;
      const _inheritedFormControlProps = pick((inheritedFormControlProps == null ? void 0 : inheritedFormControlProps.value) || {}, ["isRequired", "isDisabled", "isInvalid", "isReadOnly"]);
      const cleanedContextProps = filterUndefined({
        readonly: (_a = _inheritedFormControlProps == null ? void 0 : _inheritedFormControlProps.isReadOnly) == null ? void 0 : _a.value,
        required: (_b = _inheritedFormControlProps == null ? void 0 : _inheritedFormControlProps.isRequired) == null ? void 0 : _b.value,
        disabled: (_c = _inheritedFormControlProps == null ? void 0 : _inheritedFormControlProps.isDisabled) == null ? void 0 : _c.value,
        invalid: (_d = _inheritedFormControlProps == null ? void 0 : _inheritedFormControlProps.isInvalid) == null ? void 0 : _d.value
      });
      const cleanedOwnProps = filterUndefined({
        required: props.isRequired,
        disabled: props.isDisabled,
        invalid: props.isInvalid,
        readonly: props.isReadonly,
        focusable: props.isFocusable,
        "aria-invalid": props["aria-invalid"],
        "aria-label": props["aria-label"],
        "aria-labelledby": props["aria-labelledby"],
        "aria-describedby": props["aria-describedby"]
      });
      return mergeWith({}, cleanedContextProps, cleanedOwnProps, {
        value: props.value,
        defaultChecked: props.defaultChecked,
        indeterminate: props.isIndeterminate
      });
    });
    const context = computed(() => ({
      id,
      ids: {
        root: rootId.value,
        input: inputId.value,
        control: controlId.value,
        labelId: labelId.value
      },
      ...machineContext.value
    }));
    const [state, send] = useMachine(
      checkbox.machine({
        id: "chakra-ui-checkbox-machine"
      }),
      {
        context
      }
    );
    const api = computed(() => checkbox.connect(state.value, send, normalizeProps));
    const iconStyles = computed(() => ({
      opacity: api.value.isChecked || api.value.isIndeterminate ? 1 : 0,
      transform: api.value.isChecked || api.value.isIndeterminate ? "scale(1)" : "scale(0.95)",
      fontSize: props.iconSize,
      color: props.iconColor,
      ...styles.value.icon
    }));
    const clonedIcon = computed(() => cloneVNode(h(CheckboxIcon), {
      __css: iconStyles.value,
      isIndeterminate: api.value.isIndeterminate,
      isChecked: api.value.isChecked
    }));
    onMounted(() => {
      if (props.defaultChecked && api.value) {
        api.value.setChecked(true);
      }
      if (api.value) {
        if (group.value.value && ownProps.value.value) {
          const isChecked = group.value.value.includes(ownProps.value.value);
          api.value.setChecked(isChecked);
        }
      }
    });
    watch(() => api.value.isChecked, (value) => {
      emit("update:modelValue", value);
      emit("change", value);
      if (group.value.handleChange && ownProps.value.value) {
        group.value.handleChange(ownProps.value.value, value);
      }
    });
    watch(() => props.modelValue, (value) => {
      api.value.setChecked(value);
    });
    const inputProps = computed(() => mergeProps(api.value.inputProps, {
      onChange() {
        if (api.value.isIndeterminate && !api.value.isReadOnly) {
          api.value.setIndeterminate(false);
          api.value.setChecked(true);
        }
      }
    }));
    return () => {
      const children = getValidChildren(slots);
      const hasChildren = children.length > 0;
      return _createVNode(chakra.div, {
        "sx": {
          "> div": styles.value.container
        }
      }, {
        default: () => _createVNode(CLabel, _mergeProps({
          "__label": "checkbox"
        }, api.value.rootProps, {
          "__css": styles.value.container
        }), {
          default: () => _createVNode(_Fragment, null, [_createVNode("input", _mergeProps({
            "class": "chakra-checkbox__input"
          }, inputProps.value), null), _createVNode(CCheckboxControl, _mergeProps({
            "__label": "checkbox__control",
            "__css": styles.value.control
          }, api.value.controlProps), {
            default: () => clonedIcon.value
          }), hasChildren && _createVNode(chakra.span, _mergeProps({
            "label": "checkbox__label"
          }, api.value.labelProps, {
            "__css": {
              marginStart: props.spacing,
              ...styles.value.label
            }
          }), _isSlot(slots) ? slots : {
            default: () => [slots]
          })])
        })
      });
    };
  }
});

export {
  CCheckbox
};
